<?php

namespace App\Http\Controllers\master;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Validator;
use App\Models\AccountCategory;
use App\Models\Ledger;
use App\Services\ReportService;
use DB;
use Illuminate\Support\Facades\DB as FacadesDB;

class CategoryController extends Controller
{
    private function validateAccesToken($access_token)
    {
        return ['id' => 1, 'name' => 'ADMIN'];
    }

    public function index(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            // 'access_token' => 'required'
            'companyID' => 'required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            $page    = $request->page ?? 1;
            $limit   = $request->limit ?? 10;
            $offset  = ($page - 1) * $limit;
            $list    =  AccountCategory::select('ACC_NmeMainCategory')->where('ACC_IdCompany', $request->companyID)->distinct('ACC_NmeMainCategory')->get();
            if ($list->count() > 0) {

                $o_data['list'] = convert_all_elements_to_string($list->toArray());
                $status = "1";
                $message = "data fetched successfully";
            } else {
                $message = "no data to show for your request";
            }
        }

        return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    public function subCategory(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            'companyID' => 'required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {

            $list = AccountCategory::where('ACC_IdCompany', $request->companyID)->get();

            if ($list->count() > 0) {

                $o_data['list'] = convert_all_elements_to_string($list->toArray());
                $status = "1";
                $message = "data fetched successfully";
            } else {
                $message = "no data to show for your request";
            }
        }

        return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    public function datatable_list(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            // 'access_token' => 'required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            return $this->get_category_list($request->all());
        }

        return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    public function get_category_list($postData = null)
    {

        $response = array();


        ## Read value
        $draw = $postData['draw'];

        $start = $postData['start'];

        $rowperpage = $postData['length']; // Rows display per page

        $columnIndex = $postData['order'][0]['column']; // Column index

        $columnName = $postData['columns'][$columnIndex]['data']; // Column name

        $columnSortOrder = $postData['order'][0]['dir']; // asc or desc

        $searchValue = strtoupper($postData['search']['value']); // Search value



        $filter = [];


        ## Search 

        $searchQuery = "";

        if ($searchValue != '') {

            $searchQuery = " (ACC_IdCompany like '%" . $searchValue . "%' or ACC_IdRecord like '%" . $searchValue . "%' or ACC_NmeCategoryType like '%" . $searchValue . "%' or ACC_NmeMainCategory like '%" . $searchValue . "%' or ACC_NmeSubCategory like '%" . $searchValue . "%'  ) ";
        }



        ## Total number of records without filtering
        $list = $this->_get_account_category_list('', FALSE, $filter);
        $totalRecords = $list->get()->count();



        ## Total number of record with filtering

        $list = $this->_get_account_category_list($searchQuery, FALSE, $filter, $postData);
        $totalRecordwithFilter = $list->get()->count();



        ## Fetch records
        $list = $this->_get_account_category_list($searchQuery, FALSE, $filter, $postData);
        $list = $list->orderBy($columnName, $columnSortOrder);
        if ($rowperpage != 'all') {
            $list = $list->skip($start)->take($rowperpage);
        }
        $records = $list->get();



        $data = array();
        $types = [
            'BS' => 'Balance Sheet',
            'PL' => 'Profit & Loss'
        ];
        // work related to pdf.
        $exportPdf = false;
        $columns = [];
        $title = "";

        if (!empty($postData['export_pdf']) && $postData['export_pdf']) {
            $exportPdf = true;
            $columns = [
                "Type",
                "Main Category",
                "Sub Category"
            ];
            $title = "Accounting Categories";
        }

        foreach ($records as $record) {

            $action = '';
            if ($exportPdf) {
                $data[] = array(
                    "ACC_NmeCategoryTypeText"       => $record->ACC_NmeCategoryTypeText,
                    "ACC_NmeMainCategory"           => $record->ACC_NmeMainCategory,
                    "ACC_NmeSubCategory"            => $record->ACC_NmeSubCategory,
                );
            } else {
                $data[] = array(
                    "ACC_IdCompany"               => $record->ACC_IdCompany,
                    "ACC_IdRecord"                => $record->ACC_IdRecord,
                    "ACC_NmeCategoryType"         => $record->ACC_NmeCategoryType,
                    "ACC_NmeCategoryTypeText"     => $types[$record->ACC_NmeCategoryType],
                    "ACC_NmeMainCategory"         => $record->ACC_NmeMainCategory,
                    "ACC_NmeSubCategory"          => $record->ACC_NmeSubCategory,
                    "ACC_NmeType"                 => $record->ACC_NmeType,
                    "Action" => $action
                );
            }
        }

        if ($exportPdf) {
            $status   = "0";
            $message  = "";
            $errors   = [];
            $reportService = (new ReportService($data, $columns, $title));
            $fileLink = $reportService->handle();

            if ($fileLink) {
                $message = "Report generated successfully";
            } else {
                $message = "no data to show for your request";
            }
            return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => ['link' => $fileLink]], 200);
        }

        ## Response

        $response = array(

            "draw" => intval($draw),

            "iTotalRecords" => $totalRecords,

            "iTotalDisplayRecords" => $totalRecordwithFilter,

            "aaData" => $data

        );



        return $response;
    }

    private function _get_account_category_list($searchQuery = '', $needSql = FALSE, $filter = [], $postData = [])
    {

        // To get only one matched row
        // $list = \DB::table('CONT_AcCategory')
        //     ->select('*')
        //     ->whereIn('ACC_IdRecord', function ($query) {
        //         $query->select(\DB::raw('MIN(ACC_IdRecord)'))
        //             ->from('CONT_AcCategory')
        //             ->groupBy('ACC_NmeMainCategory');
        //     });

        $list = \DB::table('CONT_AcCategory')
        ->select('*');


        if (isset($postData['companyID'])) {
            $list = $list->where('ACC_IdCompany', $postData['companyID']);
        }
        if (isset($postData['category'])) {
            $list = $list->where('ACC_NmeMainCategory', $postData['category']);
        }
        if (isset($postData['sub_category']) && !empty($postData['sub_category'])) {
            $list = $list->where('ACC_NmeCategoryType', $postData['sub_category']);
        }


        if ($searchQuery != '') {
            $list = $list->whereRaw($searchQuery);
        }
        // $list = $list->groupBy('ACC_NmeMainCategory');
        if ($needSql) {
            return $list = $list->toSql();
        } else {
            return $list;
        }
    }
    public function storeCategory(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            // 'access_token'           => 'required',
            'category'           => 'required',
            'account_type'           => 'required',
            'sub_category.*'           => 'required',
            'companyID'          => 'required',
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            $companyID = $request->companyID;
            // 'C04';
            // $check = AccountCategory::where(function ($query) use ($request, $companyID) {
            //     $query->where(['ACC_NmeMainCategory' => strtoupper($request->category), 'ACC_IdCompany' => $companyID])
            //         ->orWhere(function ($query2) use ($request, $companyID) {
            //             $query2->where(['ACC_IdCompany' => $companyID, "ACC_NmeSubCategory" => $request->sub_category]);
            //         });
            // })->get();
            $check = collect();
            if (empty($request->record_id)) {
                foreach ($request->sub_category as $subCategory) {
                    $result = AccountCategory::where(function ($query) use ($request, $companyID, $subCategory) {
                        $query->where([
                            'ACC_NmeMainCategory' => strtoupper($request->category),
                            'ACC_IdCompany' => $companyID
                        ])->orWhere(function ($query2) use ($request, $companyID, $subCategory) {
                            $query2->where([
                                'ACC_IdCompany' => $companyID,
                                'ACC_NmeSubCategory' => $subCategory['ACC_NmeSubCategory'],
                                'ACC_NmeType' => $subCategory['ACC_NmeType']
                            ]);
                        });
                    })->get();

                    $check = $check->merge($result);
                }
            }


            if ($check->count() > 0) {
                $message = "Category already added";
            } else {
                DB::beginTransaction();
                try {
                    $data = [];
                    // dd($request->sub_category);
                    $sub_categories = AccountCategory::select('ACC_IdRecord', 'ACC_NmeMainCategory')->where('ACC_NmeMainCategory', $request->category)
                        ->where('ACC_NmeCategoryType', $request->account_type)->get()->toArray();

                    foreach ($request->sub_category as $key => $sub_category) {

                        $temp = [
                            "ACC_IdCompany" => $companyID,
                            "ACC_NmeMainCategory" => $request->category,
                            "ACC_NmeCategoryType" => $request->account_type,
                            "ACC_NmeSubCategory" => $sub_category['ACC_NmeSubCategory'],
                            "ACC_NmeType" => $sub_category['ACC_NmeType'],
                            "ACC_DtLastUpdated" => date('Y-m-d H:i:s'),
                            "ACC_NmeUserLastUpdated" => 'ADMIN'
                        ];
                        if ($request->record_id && isset($sub_category['ACC_IdRecord'])) {
                            foreach ($sub_categories as $index => $record) {
                                if ($record['ACC_IdRecord'] == $sub_category['ACC_IdRecord']) {
                                    unset($sub_categories[$index]);
                                    break; // Exit the loop once matched record is removed
                                }
                                if ($record['ACC_IdRecord'] == $request->record_id) {
                                    unset($sub_categories[$index]);
                                    break; // Exit the loop once matched record is removed
                                }
                            }
                            AccountCategory::where('ACC_IdRecord', $sub_category['ACC_IdRecord'])->update($temp);
                        } else {
                            AccountCategory::create($temp);
                        }
                    }

                    // Delete the remaining records in $sub_categories array
                    $recordsToDelete = [];
                    foreach ($sub_categories as $record) {
                        $recordsToDelete[] = $record['ACC_IdRecord'];
                    }

                    // Delete records from AccountCategory table
                    if (!empty($recordsToDelete)) {
                        AccountCategory::whereIn('ACC_IdRecord', $recordsToDelete)->delete();
                    }

                    FacadesDB::commit();
                    $status = "1";
                    $message = "Record saved successfully";
                } catch (\Exception $e) {
                    FacadesDB::rollback();
                    $duplicateString = 'duplicate'; // Change this to the string you want to check for duplication
                    $message = $e->getMessage();
                    if (strpos($message, $duplicateString) !== false) {
                        // If the duplicate string is found, modify the message
                        $message = str_replace($duplicateString, 'Duplicate entry', $message);
                    }
                    $message = "Transaction Failed: " . $message;
                }
            }
        }

        return response()->json(['status' => $status, 'errors' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    public function store(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            // 'access_token'           => 'required',
            'category'           => 'required',
            'account_type'           => 'required',
            'sub_category'           => 'required',
            'companyID'          => 'required',
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            // $companyID = 'C04';
            $companyID = $request->companyID;

            $check = AccountCategory::where(function ($query) use ($request, $companyID) {
                $query->where(['ACC_NmeMainCategory' => strtoupper($request->category), 'ACC_IdCompany' => $companyID])
                    ->orWhere(function ($query2) use ($request, $companyID) {
                        $query2->where(['ACC_IdCompany' => $companyID, "ACC_NmeSubCategory" => $request->sub_category]);
                    });
            })->get();


            if ($check->count() > 0) {
                $message = "Category already exist";
            } else {
                DB::beginTransaction();
                try {

                    $data = [
                        "ACC_IdCompany" => $companyID,
                        "ACC_NmeMainCategory" => $request->category,
                        "ACC_NmeCategoryType" => $request->account_type,
                        "ACC_NmeSubCategory" => $request->sub_category,
                        "ACC_NmeType" => 'OT',
                        "ACC_DtLastUpdated" => date('Y-m-d H:i:s'),
                        "ACC_NmeUserLastUpdated" => 'ADMIN'
                    ];

                    $category = AccountCategory::create($data);

                    DB::commit();
                    $status = "1";
                    $message = "Record saved successfully";
                } catch (\Exception $e) {
                    DB::rollback();
                    $message = "Transaction Faild: " . $e->getMessage();
                }
            }
        }

        return response()->json(['status' => $status, 'errors' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    public function update(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            'record_id'           => 'required',
            // 'access_token'           => 'required',
            'category'           => 'required',
            'account_type'           => 'required',
            'sub_category'           => 'required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {

            DB::beginTransaction();
            try {

                $data = [
                    "ACC_NmeMainCategory" => $request->category,
                    "ACC_NmeCategoryType" => $request->account_type,
                    "ACC_NmeSubCategory" => $request->sub_category,
                    "ACC_DtLastUpdated" => date('Y-m-d H:i:s'),
                ];

                $category = AccountCategory::where('ACC_IdRecord', $request->record_id)->update($data);

                DB::commit();
                $status = "1";
                $message = "Record updated successfully";
            } catch (\Exception $e) {
                DB::rollback();
                $message = "Transaction Faild: " . $e->getMessage();
            }
        }

        return response()->json(['status' => $status, 'errors' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }

    function edit($id)
    {
        $category = AccountCategory::where('ACC_IdRecord', $id)->first()->toArray();
        $sub_category = AccountCategory::where('ACC_NmeMainCategory', $category['ACC_NmeMainCategory'])
            ->where('ACC_NmeCategoryType', $category['ACC_NmeCategoryType'])->get()->toArray();
        // Iterate through the result and add the 'id' key
        foreach ($sub_category as $key => &$item) {
            $item['id'] = $key + 1; // Add a new key 'id' with the value of 'ACC_IdRecord'
        }
        $category['sub_category'] = $sub_category;
        return $category;
    }

    public function delete(REQUEST $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            // 'access_token' => 'required',
            'ACC_IdRecord' => 'required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();
        } else {
            $check = AccountCategory::where(['ACC_IdRecord' => strtoupper($request->ACC_IdRecord)])->first();
            if ($check) {
                DB::beginTransaction();
                try {
                    $check_vat_region = Ledger::where(['led_ledgergroup' => $check->ACC_IdRecord, 'led_compcode' => $check->ACC_IdCompany])->get()->count();
                    if ($check_vat_region > 0) {
                        $message = "You cant delete this record its associated with other record";
                    } else {
                        $sub_category = AccountCategory::where('ACC_NmeMainCategory', $check->ACC_NmeMainCategory)
                            ->where('ACC_NmeCategoryType', $check->ACC_NmeCategoryType);

                        AccountCategory::where(['ACC_IdRecord' => strtoupper($request->ACC_IdRecord)])->delete();
                        $sub_category->delete();
                        DB::commit();
                        $status = "1";
                        $message = "Category deleted successfully";
                    }
                } catch (\Exception $e) {
                    DB::rollback();
                    $message = "Transaction Faild: " . $e->getMessage();
                }
            } else {
                $message = "record not found";
            }
        }

        return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
    }
}
