<?php

namespace App\Http\Controllers\master;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\CountryModel;
use App\Models\LookupTableData;
use App\Models\CommodityOriginModel;
use App\Models\CountryVatRegionModel;
use App\Models\Destination;
use App\Models\Address;
use App\Models\PartyModel;
use App\Models\BankDetails;
use App\Models\DeliveryPlanAddress;
use App\Models\StoreLocation;
use App\Models\TradeModel;
use App\Models\ZCSFormControl;
use Validator;
use DB;

class SideBarController extends Controller
{

    public function getPages()
    {
        $data = ZCSFormControl::all();
        $response = [
            "status" => "1",
            'error' => "",
            "message" => "",
            "oData" => $data,
        ];
        return response()->json($response);
    }

    public function get_menu(REQUEST $request)
    {
        $errors   = [];
        $message  = "";
        $menu_list = DB::connection('sqlsrv')->select('exec sp_Web_SystemModuleProc ?, ?, ?', ['LIST', null, '<DATA><SYSYEMMODULE UserId="ADMIN" CompanyId="C02" AppId="WEB-WEBAPP"></SYSYEMMODULE></DATA>']);
        $menu_parents = [];
        $screenData = ZCSFormControl::all();

        // Find the index of the first "Administration" menu item
        $administrationIndex = null;
        foreach ($menu_list as $index => $item) {
            if ($item->OptionName === 'Administration') {
                $administrationIndex = $index;
                break;
            }
        }

        // If there is more than one "Administration" menu item, remove the second one and rename it to "Definitions"
        if ($administrationIndex !== null) {
            $secondAdministrationIndex = null;
            foreach ($menu_list as $index => $item) {
                if ($index !== $administrationIndex && $item->OptionName === 'Administration') {
                    $secondAdministrationIndex = $index;
                    break;
                }
            }

            if ($secondAdministrationIndex !== null) {
                // Rename the second "Administration" menu item to "Definitions"
                $menu_list[$secondAdministrationIndex]->OptionName = 'Definitions';

                // Merge the submenu levels of "Definitions" with the first "Administration"
                $menu_list[$administrationIndex]->childs = array_merge(
                    $menu_list[$administrationIndex]->childs ?? [],
                    $this->generateSubLevels($menu_list, $menu_list[$secondAdministrationIndex]->ChildGroupKey)
                );

                // Remove the second "Administration" menu item
                unset($menu_list[$secondAdministrationIndex]);
            }
        }

        // Filter out parent menu items
        foreach ($menu_list as $i => $k) {
            if (in_array($k->OptionDisplayType, ['P', 'S', 'I']) && $k->OptionGroupKey == "1") {
                $menu_parents[] = $k;
            }
        }

        // Generate submenu levels
        foreach ($menu_parents as $index => $parent) {
            $ChildGroupKey = $parent->ChildGroupKey;
            $parent->childs = $this->generateSubLevels($menu_list, $ChildGroupKey);
        }

        $status = "1";
        return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => $menu_parents, "screenData" => $screenData], 200);
    }

    function generateSubLevels($menu_list, $ChildGroupKey)
    {
        $result = [];
        // Filter out child menu items
        foreach ($menu_list as $item) {
            if ($item->OptionGroupKey == $ChildGroupKey) {
                $childItem = (object)[
                    'OptionName' => str_replace('VAT', 'VATCAPTION', $item->OptionName),
                    'childs' => $this->generateSubSubLevels($menu_list, $item->ChildGroupKey)
                ];
                $result[] = $childItem;
            }
        }
        return $result;
    }

    function generateSubSubLevels($menu_list, $ChildGroupKey)
    {
        $result = [];
        // Filter out child submenu items
        foreach ($menu_list as $item) {
            if ($item->OptionGroupKey == $ChildGroupKey) {
                $subChildItem = (object)[
                    'OptionName' => str_replace('VAT', 'VATCAPTION', $item->OptionName),
                    'childs' => [] // No more levels, so set childs as empty array
                ];
                $result[] = $subChildItem;
            }
        }
        return $result;
    }
}
