<?php

namespace App\Services;

use Lcobucci\JWT\Signer\Key\InMemory;
use Lcobucci\JWT\Validation\Constraint\SignedWith;
use Lcobucci\JWT\Validation\Constraint\ValidAt;
use DateTimeImmutable;
use Lcobucci\JWT\Configuration;

class JwtService
{
    private Configuration $config;

    public function __construct(string $secretKey)
    {
        $this->config = Configuration::forSymmetricSigner(
            new InMemory($secretKey)
        );
    }

    public function generateToken(array $claims): string
    {
        $builder = $this->config->builder()
            ->issuedBy('your_app') // Issuer
            ->permittedFor('your_domain') // Audience
            ->issuedAt(new DateTimeImmutable()) // Issued at
            ->canOnlyBeUsedAfter(new DateTimeImmutable()) // Start time
            ->expiresAt(new DateTimeImmutable('+1 hour')); // Expiration time

        foreach ($claims as $name => $value) {
            $builder->withClaim($name, $value);
        }

        return (string) $builder->getToken($this->config->signer(), $this->config->signingKey());
    }

    public function validateToken(string $token): bool
    {
        $token = $this->config->parser()->parse($token);

        $constraints = [
            new SignedWith($this->config->signer(), $this->config->signingKey()),
            new ValidAt(new DateTimeImmutable())
        ];

        return $this->config->validator()->validate($token, ...$constraints);
    }

    public function getClaims(string $token): array
    {
        return $this->config->parser()->parse($token)->claims()->all();
    }
}
